#!/usr/bin/python3
# -*- coding: iso-8859-1 -*-
"""
Programme du couplage entre HOMARD et Code_Aster

Copyright EDF-R&D 2002, 2003, 2005, 2006, 2012
"""
__revision__ = "V2.8"
#
from aster_homard_py import aster_homard
import os
#
class ASTER_HOMARD :
#
  """
  Classe du couplage entre ASTER et HOMARD.
  """
#
#
#====
# 1. Le constructeur
#====
#
#
  def __init__ ( self, liste_option ) :
#
    """
    """
#
#==================================================================================================#
#
# rep_homard_ref_defaut :
# repertoire contenant l'executable HOMARD, sous ses differentes versions, dans un sous-repertoire
# identifie au systeme.
# Ex : /home/aster/outils/HOMARD/Linux64/HOMARD_V10.4.out
# Seule cette variable est a parametrer pour une nouvelle installation.
    rep_homard_ref_defaut = os.path.normpath(os.path.dirname(os.path.dirname(os.path.abspath(os.path.realpath(sys.argv[0])))))
#
#==================================================================================================#
#
    self.rep_homard_ref = rep_homard_ref_defaut
#
    self.liste_option = liste_option
#
    self.verbose = 0
#
#====
# 2. La version
#====
#
#
  def lit_homard_version (self) :
#
    """
    Reperage de la version HOMARD
    """
#
    code_erreur = 0
    message_info = "Recherche de la version de HOMARD\n"
    message_erreur = ""
    hom_vers = None
    version_perso = 0
#
#    if self.verbose :
#      print self.liste_option
#      print len(self.liste_option)
#
    if len(self.liste_option) < 1 :
      code_erreur = 1
      message_erreur = "Impossible de trouver le numero de version voulu pour HOMARD."
    elif len(self.liste_option) > 2 :
      hom_vers = self.liste_option[1]
      if hom_vers[0] != "V" :
        hom_vers = self.liste_option[2]
      if len(self.liste_option) > 4 :
        version_perso = int(self.liste_option[4])
    else :
      hom_vers = "V9.x"
#
    self.homard_version = hom_vers
    self.version_perso = version_perso
#
    return code_erreur, message_info, message_erreur
#
#
#====
# 3. Lancement
#====
#
#
  def lancement (self) :
#
    """
    """
#
# 1. ==> Initialisations
#
    code_erreur = None
    message_erreur = ""
#
    homard_version_type = self.homard_version[0:2]
    if self.verbose :
      message_info = ".. Utilisation de HOMARD version " + self.homard_version + "\n"
    else :
      message_info = ""
#
# 2. ==> Les arguments
#
    arguments = self.rep_homard_ref
    for option in self.liste_option[0:4] :
      arguments += " " + option
    if self.version_perso :
      arguments += " -PERSO"
    if self.verbose :
      arguments += " -verbose"
      message_info += ".. Arguments transmis au programme aster_homard :"
      l_aux = arguments.split()
      for argu in l_aux :
        message_info += "\n    " + argu
      message_info += "\n"
#
# 3. ==> Lancement
#
    if ( homard_version_type in [ "V3", "V4", "V5", "V6", "V7", "V8" ] ) :
      message_erreur  = "Ces versions de HOMARD sont obsoletes.\n"
      message_erreur += "Nous vous conseillons de passer avec une version plus recente de Code_Aster."
    else :
      from aster_homard_py import aster_homard
      code_erreur, message_info, message_erreur = aster_homard.aster_homard (arguments, message_info)
#
# 4. ==> C'est fini
#
    if ( homard_version_type in [ "V9" ] ) :
      if code_erreur :
        message_erreur  = "Erreur au lancement de HOMARD.\n"
        message_erreur += "Nous vous conseillons de passer avec une version plus recente de Code_Aster."
#
    return code_erreur, message_info, message_erreur
#
#
#=======================================================================================
#=======================================================================================
#
#
if __name__ == "__main__" :
#
  import sys
#
# 1. ==> Recuperation des arguments
#
  liste_option_main = sys.argv[1:]
#
# 2. ==> Creation de la classe
#
  ASTER_HOMARD = ASTER_HOMARD(liste_option_main)
#
# 3. ==> La version
#
  code_erreur_main, message_info_main, message_erreur_main = ASTER_HOMARD.lit_homard_version()
#
# 4. ==> Lancement
#
  if not code_erreur_main :
    code_erreur_main, message_info_main, message_erreur_main = ASTER_HOMARD.lancement()
#
# 5. ==> La fin
#
## Solution temporaire, en attente de la gestion du stderr
##
  if ( code_erreur_main ) :
    message_info_main += message_erreur_main
  sys.stdout.write(message_info_main)
  sys.stderr.write(message_erreur_main)
  sys.exit(code_erreur_main)
